import java.util.*;

abstract public class M865Tree extends DWGraph {

	// A tree is a connected graph with no cycles.
	// These conditions must be enforced in every constructor, accessor,
	// and alteration method.

	protected boolean updateCertificate;	// flag for updating the certificate
	protected String certificate;				// the certificate for the tree
	protected boolean updateRootedTree;		// flag for updating the rooted tree
	protected M865TreeNode root;				// a rooted linked form of the tree
	protected TreeMap nodeOf;					// The map connecting the vertices with their Tree Nodes.
	protected M865Vertex center;				// The center node or the first of the two central nodes.


	public M865Tree(M865Tree tree) {
		super(tree.getVertexSet(), tree.getEdgeList());
		updateCertificate = true;
		certificate = "";
		updateRootedTree = true;
		root = null;
		nodeOf = null;
	}
	
	public M865Tree(M865Graph graph) {
		super(graph);
		// A tree is a connected graph with no cycles.
		if (!graph.isConnected() || graph.hasCycles()) {
			System.out.println(graph);
			throw (new UnsupportedOperationException("Graph is not a tree"));
		}
		updateCertificate = true;
		certificate = "";
		updateRootedTree = true;
		root = null;
		nodeOf = null;
	}

	public M865Tree(SortedSet vertexSet, SortedSet edgeList) {
		this(new DWGraph(vertexSet, edgeList));
	}
		
	public M865Tree() {
		this(new TreeSet(), new TreeSet());
	}
	
	public boolean addVertex(M865Vertex v) {
		if (vertexSet.size() == 0) {
			super.addVertex(v);
			updateCertificate = true;
			updateRootedTree = true;
			return true;
		} else if (containsVertex(v)) {
			return false;
		} else {
			throw (new UnsupportedOperationException("Cannot add an isolated vertex to an existing tree"));
		}
	}

	public boolean removeVertex(M865Vertex leaf) {
		int deg = degreeOf(leaf);
		if (deg == 0) {
			return false;
		} else if (deg == 1) {			
			super.removeVertex(leaf);
			updateCertificate = true;
			updateRootedTree = true;
			return true;
		} else {
			throw (new UnsupportedOperationException("Can only remove leaves"));
		}
	}

	public boolean addEdge(M865Edge e) {
		if (containsEdge(e)) {
			return false;
		} else {
			M865Vertex s = e.getSource();
			M865Vertex t = e.getTarget();
			if (vertexSet.contains(s) && vertexSet.contains(t)) {
				throw (new UnsupportedOperationException("Cannot add cycles"));
			} else if (vertexSet.contains(s) || vertexSet.contains(t)) {
				super.addEdge(e);
				updateCertificate = true;
				updateRootedTree = true;
				return true;
			} else {
				throw (new UnsupportedOperationException("Cannot add an isolated edge"));
			}
		}
	}

	public boolean removeEdge(M865Edge e) {
		if (!containsEdge(e)) {
			return false;
		} else {
			M865Vertex s = e.getSource();
			M865Vertex t = e.getTarget();
			if (degreeOf(s) == 1) {
				removeVertex(s);
			} else if (degreeOf(t) ==1) {
				removeVertex(t);
			} else {
				throw (new UnsupportedOperationException("Cannot remove an internal edge"));
			}
			return true;
		}
	}

	public int getNumberOfComponents() {
		// A tree is a connected graph with no cycles, hence only one component.
		if (super.getNumberOfComponents() != 1) {
			throw( new ConcurrentModificationException("This M865Tree has been altered"));
		}
		return 1;
	}
	
	public boolean isConnected() {
		// A tree is a connected graph with no cycles.
		if (super.getNumberOfComponents() != 1) {
			throw( new ConcurrentModificationException("This M865Tree has been altered"));
		}
		return true;
	}

	public boolean hasCycles() {
		// A tree is a connected graph with no cycles.
		if (cyclic) {
			throw( new ConcurrentModificationException("This M865Tree has been altered"));
		}
		return false;
	}

	public M865Graph getSpanningTree() {
		return this;
	}


	abstract public String getCertificate();

	abstract public M865Vertex getCenter();

	abstract public M865TreeNode buildRootedTree(M865Vertex root);


	public String toString() {
		String s = "M865Tree (\n   Vertex Set: ";
		s += vertexSet.toString() + "\n   Edge List: ";
		s += edgeList.toString() + ")";
		return s;
	}	
}	
